#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2010, 2011 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import sys
import os.path
from skoolkit.skoolctl import CtlWriter, BLOCKS, BLOCK_TITLES, BLOCK_DESC, REGISTERS, BLOCK_COMMENTS, SUBBLOCKS, COMMENTS

def parse_args(args):
    elements = BLOCKS + BLOCK_TITLES + BLOCK_DESC + REGISTERS + BLOCK_COMMENTS + SUBBLOCKS + COMMENTS
    write_hex = False
    write_asm_dirs = True
    files = []
    i = 0
    while i < len(args):
        arg = args[i]
        if arg[:2] == '-w':
            elements = arg[2:]
        elif arg == '-h':
            write_hex = True
        elif arg == '-a':
            write_asm_dirs = False
        elif len(arg) > 1 and arg[0] == '-':
            print_usage()
        else:
            files.append(arg)
        i += 1

    if len(files) != 1:
        print_usage()
    return files[0], elements, write_hex, write_asm_dirs

def print_usage():
    sys.stderr.write("""Usage: %s [options] FILE

  Convert a skool file into a control file, written to standard output. FILE
  may be a regular file, or '-' for standard input.

Options:
  -wX  Write only these elements, where X is one or more of:
         %s = block types and addresses
         %s = block titles
         %s = block descriptions
         %s = registers
         %s = mid-block comments and block end comments
         %s = sub-block types and addresses
         %s = instruction-level comments
  -h   Write addresses in hexadecimal format
  -a   Do not write ASM directives
""" % (os.path.basename(sys.argv[0]), BLOCKS, BLOCK_TITLES, BLOCK_DESC, REGISTERS, BLOCK_COMMENTS, SUBBLOCKS, COMMENTS))
    sys.exit(1)

def run(skoolfile, elements, write_hex, write_asm_dirs):
    writer = CtlWriter(skoolfile, elements, write_hex, write_asm_dirs)
    writer.write()

def main():
    run(*parse_args(sys.argv[1:]))

if __name__ == '__main__':
    main()
