# -*- coding: utf-8 -*-
from StringIO import StringIO
import unittest
from skoolkittest import SkoolKitTestCase
from skoolkit.skoolsft import SftWriter

TEST_SKOOL = 'test-sft.skool'

TEST_SFT = """; Test skool file for SftWriter unit testing

; Dangling comment not associated with any entry

; Data definition entry
d49152 DEFB 0 ; Comment 1
 49153 DEFW 0 ; Comment 2

; Remote entry
r24576 other
 24579

; @start
; @org=32768

; Routine
;
; @ignoredua
; Routine description
;
; A Some value
; B Another value
; @label=START
; @isub=DI
cC32768,1;20 Do nothing
; @bfix=DEFB 1,3
 B32769,2;20 1-line B sub-block
; @ignoreua
 B32771,1;20 {2-line B sub-block
; @ssub=DEFB 5,6
 B32772,2;20 }
; @ignoremrcua
; Mid-block comment
 T32774,5;20 T sub-block
; @keep
 W32779,2;20 W sub-block
; @nowarn
 Z32781,2;20 Z sub-block
; @nolabel
; @ofix=LD A,6
*C32783,2;20 {Sub-block with instructions of various types
; @rem=Hello!
 B32785,1;20
; @rsub=DEFB 3
 W32786,4;20
 T32790,2;20
 Z32792,3;20 }
 C32795,1;20 Return
 ;20 comment continuation line
; End comment paragraph 1.
; .
; End comment paragraph 2.

; Test ASM block directives
bB32796,1
; @bfix-begin
 B32797,1
; @bfix+else
       DEFB 101
; @bfix+end
 B32798,1
; @isub+begin
       DEFB 102
; @isub-else
 B32799,1
; @isub-end
; @ofix-begin
 B32800,1
; @ofix+else
 32800 DEFB 104
; @ofix+end
; @rfix+begin
       DEFB 205
; @rfix+end
; @rsub-begin
 B32802,1
; @rsub-end

; Ignore block
i32803 DEFB 56  ; Set 32803 to 56
 32804 DEFW 512

; Data block
bB49152,1

; Game status buffer entry
gW49153,2

; Message
tT49155,2

; Unused block
uB49157,1

; Word block
wW49158,2*2

; Zero block
zZ49162,10
; @end

; Another ignore block
i49172""".split('\n')

class SftWriterTest(SkoolKitTestCase):
    def _get_sft(self, write_hex=False):
        writer = SftWriter(TEST_SKOOL, write_hex)
        out = StringIO()
        writer.write(out)
        return out.getvalue().split('\n')[:-1]

    def test_sftwriter(self):
        sft = self._get_sft()
        self.assertEqual(len(sft), len(TEST_SFT))
        for i, line in enumerate(sft):
            self.assertEqual(line, TEST_SFT[i])

if __name__ == '__main__':
    unittest.main()
